@@ -11,21 +11,22 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
+;; XEmacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the Free
-;; Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
-
-;;; Synched up with: cal-menu.el in Emacs 20.2
+;; along with XEmacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
+ 
+;;; Synched up with: cal-menu.el in Emacs 20.3
 
 ;;; Commentary:
 
@@ -40,9 +41,9 @@
 
 ;;; Code:
 
-;;(define-key calendar-mode-map 'button2 'calendar-mouse-2-date-menu)
-;;(define-key calendar-mode-map 'button2up 'ignore)
-
+(eval-when-compile (require 'calendar))
+(require 'easymenu)
+ 
 (defconst calendar-popup-menu-3
   '("Calendar"
     ["Scroll forward" scroll-calendar-left-three-months t]
@@ -65,14 +66,14 @@
 
 (defun calendar-popup-menu-2 (e)
   (interactive "@e")
-  (setq calendar-foobar (calendar-event-to-date e t))
+  (setq calendar-foobar (calendar-event-to-date  t))
   (let ((menu (list (format "Menu - %s" (calendar-date-string calendar-foobar) t t)
 	       "-----"
 	       ["Holidays" calendar-mouse-holidays t]
 	       ["Mark date" calendar-mouse-set-mark t]
 	       ["Sunrise/sunset" calendar-mouse-sunrise/sunset t]
-	       ["Other calendars" calendar-mouse-print-dates (calendar-event-to-date e)]
-	       ["Prepare LaTeX Buffer" calendar-mouse-cal-tex-menu (calendar-event-to-date e)]
+	       ["Other calendars" calendar-mouse-print-dates (calendar-event-to-date )]
+	       ["Prepare LaTeX Buffer" calendar-mouse-cal-tex-menu (calendar-event-to-date )]
 	       ["Diary entries" calendar-mouse-view-diary-entries t]
 	       ["Insert diary entry" calendar-mouse-insert-diary-entry t]
 	       ["Other Diary file entries"
@@ -83,31 +84,14 @@
 
 (define-key calendar-mode-map 'button2 'calendar-popup-menu-2)
 
-(defun calendar-mouse-cal-tex-menu (e) 
-  "Pop up submenu for Mouse-2 for cal-tex commands for selected date in the calendar window."
-  (interactive "e")
-   (let* ((menu (list (calendar-date-string calendar-foobar t nil)
-		     "-----"
-		     ["Daily (1 page)"    cal-tex-mouse-day  ]
-		     ["Weekly (1 page)"   cal-tex-mouse-week ]
-		     ["Weekly (2 pages)"  cal-tex-mouse-week2]
-		     ["Weekly (other style; 1 page)" cal-tex-mouse-week-iso]
-		     ["Weekly (yet another style; 1 page)"
-		      cal-tex-mouse-week-monday]
-		     ["Monthly" cal-tex-mouse-month]
-		     ["Monthly (landscape)"  cal-tex-mouse-month-landscape]
-		     ["Yearly"  cal-tex-mouse-year]
-		     ["Yearly (landscape)"  cal-tex-mouse-year-landscape]
-		     ["Filofax styles" cal-tex-mouse-filofax]
-		     )))
-    (popup-menu menu)
-    ))
 
 (defun cal-tex-mouse-filofax (e) 
   "Pop up sub-submenu for Mouse-2 for Filofax cal-tex commands for selected date."
   (interactive "e")
    (let* ((menu (list (calendar-date-string calendar-foobar t nil)
 		     "-----"
+		     ["Filofax Daily (One-day-per-page)"   
+		      cal-tex-mouse-filofax-daily]
 		     ["Filofax Weekly (2-weeks-at-a-glance)"   
 		      cal-tex-mouse-filofax-2week]
 		     ["Filofax Weekly (week-at-a-glance)" 
@@ -214,12 +198,70 @@
   (let ((year (1- (extract-calendar-year (calendar-cursor-to-date)))))
     (list-holidays year year)))
 
-(defun calendar-event-to-date (event &optional error)
+(defun cal-menu-update ()
+  ;; Update the holiday part of calendar menu bar for the current display.
+  (interactive)
+  (condition-case nil
+      (if (eq major-mode 'calendar-mode)
+          (let ((l))
+	    (let ((date (calendar-cursor-to-date)))
+              (if date
+		  (setq l (cons (vector
+				 (format "For Date (%s)"
+					 (calendar-date-string date)) 
+				 'cal-menu-today-holidays t) l))))
+	    (let ((title
+                   (let ((m1 displayed-month)
+                         (y1 displayed-year)
+                         (m2 displayed-month)
+                         (y2 displayed-year))
+                     (increment-calendar-month m1 y1 -1)
+                     (increment-calendar-month m2 y2 1)
+                     (if (= y1 y2)
+                         (format "%s-%s, %d"
+                                 (calendar-month-name m1 3)
+                                 (calendar-month-name m2 3)
+                                 y2)
+                       (format "%s, %d-%s, %d"
+                               (calendar-month-name m1 3)
+                               y1
+                               (calendar-month-name m2 3)
+                               y2)))))
+	      (setq l (cons (vector
+			     (format "For Window (%s)" title )
+			     'list-calendar-holidays t) l)))
+	    (setq l (cons (vector
+			   (format "For Today (%s)"
+				   (calendar-date-string 
+				    (calendar-current-date) t t))
+			   'cal-menu-today-holidays t) l))
+	    (setq l (cons "---" l))
+	    (calendar-for-loop;; Show 11 years--5 before, 5 after year of
+	     ;; middle month
+             i from (- displayed-year 5) to (+ displayed-year 5) do
+             (setq l (cons (vector (format "For Year %s" i)
+                                   (list (list 'lambda 'nil '(interactive)
+                                               (list 'list-holidays i i)))
+                                   t)
+                           l)))
+            (setq l (cons ["Mark Holidays" mark-calendar-holidays t]
+                          (cons ["Unmark Calendar" calendar-unmark t]
+                                (cons ["--" '("--") t] l))))
+            (easy-menu-change '("Calendar") "Holidays" (nreverse l))
+	    ))
+    (error (ding))))
+
+(defun calendar-event-to-date (&optional error)
   "Date of last event.
 If event is not on a specific date, signals an error if optional parameter
 ERROR is t, otherwise just returns nil."
   (save-excursion
-    (goto-char (event-point event))
+    ;;  Emacs has this.  
+    ;;(set-buffer (window-buffer (posn-window (event-start last-input-event))))
+    ;;(goto-char (posn-point (event-start last-input-event)))
+    ;; I think this does the same thing
+    (set-buffer (event-buffer last-input-event))
+    (goto-char (event-point last-input-event))
     (calendar-cursor-to-date error)))
 
 (defun calendar-mouse-insert-hebrew-diary-entry (event)
@@ -254,34 +296,70 @@ ERROR is t, otherwise just returns nil."
     (setq calendar-foobar nil)
     (calendar-sunrise-sunset)))
 
-(defun calendar-mouse-holidays ()
-  "Show holidays for mouse-selected date."
+(defun cal-menu-today-holidays ()
+  "Show holidays for today's date."
   (interactive)
   (save-excursion
-    (calendar-goto-date calendar-foobar)
-    (setq calendar-foobar nil)
+    ;(calendar-cursor-to-date (calendar-current-date))
+    (calendar-goto-today)
     (calendar-cursor-holidays)))
 
+(defun calendar-mouse-holidays ()
+  "Pop up menu of holidays for mouse selected date."
+  (interactive)
+  (save-excursion
+    (let* ((date calendar-foobar)
+	   (l (check-calendar-holidays date))
+	   (menu 
+	    (cons
+	     (format "Holidays for %s" (calendar-date-string date))
+	     (if l l '("None")))))
+      (setq calendar-foobar nil)
+      (popup-menu menu))))
+  
 (defun calendar-mouse-view-diary-entries ()
-  "View diary entries on mouse-selected date."
+  "Pop up menu of diary entries for mouse selected date."
+  (interactive)
+  (save-excursion
+    (let* ((date calendar-foobar)
+	   (l (mapcar '(lambda (x) (concat (car (cdr x)) ""))
+		      (let ((diary-list-include-blanks nil)
+			    (diary-display-hook 'ignore))
+			(list-diary-entries date 1))))
+	   (menu 
+	    (cons
+	     (format "Diary Entries for %s" (calendar-date-string date))
+	     (if l l '("None")))))
+      (setq calendar-foobar nil)
+      (popup-menu menu))))
+
+(defun calendar-mouse-view-other-diary-entries ()
+  "Pop up menu of diary entries from alternative file on mouse-selected date."
+  (interactive)
+  (save-excursion
+    (let* ((date calendar-foobar)
+	   (l (mapcar '(lambda (x) (concat (car (cdr x)) ""))
+		      (let ((diary-list-include-blanks nil)
+			    (diary-display-hook 'ignore)
+			    (diary-file (read-file-name
+					 "Enter diary file name: "
+					 default-directory nil t)))
+			(list-diary-entries date 1))))
+	   (menu 
+	    (cons
+	     (format "Diary Entries form %s for %s" 
+		     diary-file
+		     (calendar-date-string date))
+	     (if l l '("None")))))
+      (setq calendar-foobar nil)
+      (popup-menu menu))))
+
+(defun calendar-mouse-insert-diary-entry ()
+  "Insert diary entry for mouse-selected date."
   (interactive)
   (save-excursion
     (calendar-goto-date calendar-foobar)
     (setq calendar-foobar nil)
-    (view-diary-entries 1)))
-
-(defun calendar-mouse-view-other-diary-entries (event)
-  "View diary entries from alternative file on mouse-selected date."
-  (interactive "e")
-  (save-excursion
-    (calendar-goto-date calendar-foobar)
-    (call-interactively 'view-other-diary-entries)))
-
-(defun calendar-mouse-insert-diary-entry (event)
-  "Insert diary entry for mouse-selected date."
-  (interactive "e")
-  (save-excursion
-    (calendar-goto-date calendar-foobar)
     (insert-diary-entry nil)))
 
 (defun calendar-mouse-set-mark ()
@@ -335,6 +413,14 @@ Holidays are included if `cal-tex-holidays' is t."
     (setq calendar-foobar nil)
     (cal-tex-cursor-week-monday nil)))
 
+(defun cal-tex-mouse-filofax-daily ()
+  "Day-per-page Filofax calendar for week indicated by cursor."
+  (interactive)
+  (save-excursion
+    (calendar-goto-date calendar-foobar)
+    (setq calendar-foobar nil)
+    (cal-tex-cursor-filofax-daily nil)))
+
 (defun cal-tex-mouse-filofax-2week ()
   "One page Filofax calendar for week indicated by cursor."
   (interactive)
@@ -427,10 +513,42 @@ The output is in landscape format, one month to a page."
 			     (calendar-mayan-date-string calendar-foobar))
 		     )))
     (popup-menu menu)
-    (message "%s" menu)
     )
   (setq calendar-foobar nil))
 
+(defun calendar-mouse-cal-tex-menu (e) 
+  "Pop up submenu for Mouse-2 for cal-tex commands for selected date in the calendar window."
+  (interactive "e")
+   (let* ((menu (list (calendar-date-string calendar-foobar t nil)
+		     "-----"
+		     ["Daily (1 page)"    cal-tex-mouse-day  ]
+		     ["Weekly (1 page)"   cal-tex-mouse-week ]
+		     ["Weekly (2 pages)"  cal-tex-mouse-week2]
+		     ["Weekly (other style; 1 page)" cal-tex-mouse-week-iso]
+		     ["Weekly (yet another style; 1 page)"
+		      cal-tex-mouse-week-monday]
+		     ["Monthly" cal-tex-mouse-month]
+		     ["Monthly (landscape)"  cal-tex-mouse-month-landscape]
+		     ["Yearly"  cal-tex-mouse-year]
+		     ["Yearly (landscape)"  cal-tex-mouse-year-landscape]
+		     ["Filofax styles" cal-tex-mouse-filofax]
+		     )))
+    (popup-menu menu)
+    ))
+
+(defun calendar-mouse-chinese-date ()
+  "Show Chinese equivalent for mouse-selected date."
+  (interactive)
+  (save-excursion
+    (calendar-goto-date calendar-foobar)
+    (setq calendar-foobar nil)
+    (calendar-print-chinese-date)))
+
+(defun calendar-mouse-goto-date (date)
+  (set-buffer (event-buffer  last-input-event))
+  (calendar-goto-date date))
+
+
 (run-hooks 'cal-xemacs-load-hook)
 
 (provide 'cal-xemacs)
